#include "DHDCC.h"

// ---------------------------------------------------------------------------------------------------------------------------------------------------------------
// CVs config 
// According to NMRA S-9.2.2 we must have only three mandatory CVs. DHDCC_CV1 DHDCC_ADDress, DHDCC_CV7 Mfr version, DHDCC_CV8 Mfr Id.
// ---------------------------------------------------------------------------------------------------------------------------------------------------------------

volatile byte DHDCC_CV1;
volatile byte DHDCC_CV7;
volatile byte DHDCC_CV8;
volatile byte DHDCC_CV29;

// ---------------------------------------------------------------------------------------------------------------------------------------------------------------
// Service mode operations
// ---------------------------------------------------------------------------------------------------------------------------------------------------------------
volatile bool DHDCC_ServiceMode;

// ---------------------------------------------------------------------------------------------------------------------------------------------------------------
// packet catcher
// ---------------------------------------------------------------------------------------------------------------------------------------------------------------

volatile int DHDCC_StartTime;
volatile int DHDCC_Interval;

// parser stage
volatile byte DHDCC_ParserStage;

#define DHDCC_WaitPreamble        1
#define DHDCC_WaitPacketStartBit  2
#define DHDCC_ReadDHDCC_ADDress         3
#define DHDCC_DHDCC_WaitDataStartBit    4
#define DHDCC_ReDHDCC_ADData            5
#define DHDCC_WaitPacketEndBit    6 

// preamble processing
volatile byte DHDCC_PreambleBitsCounter;

// packet processing
volatile bool DHDCC_WaitData;
// Packet format DHDCC_TempPacket[0] - DHDCC_ADDress byte, DHDCC_TempPacket[1..DHDCC_MaxPacketLength] - data bytes  
volatile byte DHDCC_TempPacket[DHDCC_MaxPacketLength];
volatile byte DHDCC_TempPacketLength=0;
volatile byte DHDCC_DataBitsCounter;
volatile byte DHDCC_DatabytesCounter;

// DHDCC_Reset packet counter
volatile byte DHDCC_DHDCC_ResetPacketsCounter;

// DHDCC_Reset DHDCC_Idle DHDCC_Stop flags
volatile bool DHDCC_Reset;
volatile bool DHDCC_Stop;
volatile bool DHDCC_Idle;


// ---------------------------------------------------------------------------------------------------------------------------------------------------------------
// DCC Packet stack 
// ---------------------------------------------------------------------------------------------------------------------------------------------------------------


DCCPacketStack PacketStack;


// ---------------------------------------------------------------------------------------------------------------------------------------------------------------
// class for holding DCC packet
// ---------------------------------------------------------------------------------------------------------------------------------------------------------------

DCCPacket::DCCPacket(void){
  DHDCC_ADDress=0;
  DataPositionCounter=0;
}

void DCCPacket::SetDHDCC_ADDress(volatile byte * InDHDCC_ADDress){
   DHDCC_ADDress=*InDHDCC_ADDress;
};

bool DCCPacket::PushData(volatile byte * InData){
    
  if(DataPositionCounter<(DHDCC_MaxPacketLength-1)){
    Data[DataPositionCounter]=*InData;
    DataPositionCounter++;
    return true;
  }
    return false;
}
byte DCCPacket::GetDHDCC_ADDress(void){
  return DHDCC_ADDress;
};

byte DCCPacket::GetDataLength(void){
  return DataPositionCounter;
};

byte DCCPacket::GetData(byte Position){
  return Data[Position];
};

// ---------------------------------------------------------------------------------------------------------------------------------------------------------------
// class for holding DCC packets stack
// ---------------------------------------------------------------------------------------------------------------------------------------------------------------
DCCPacketStack::DCCPacketStack(void){
  StoredPackets=0;
  LastStoredPacketPosition=1;
};

bool DCCPacketStack::PushPacket(DCCPacket * InPacket){
  if(LastStoredPacketPosition<10){
    LastStoredPacketPosition++;
    Packets[LastStoredPacketPosition-1]=*InPacket;
    return true;
  }
  return false;
};

bool DCCPacketStack::PopPacket(DCCPacket * OutPacket){

  if(OutPacket!=NULL && LastStoredPacketPosition>1){
    //Serial.print("lp=");
    //Serial.println(LastStoredPacketPosition-1,DEC);
    *OutPacket=Packets[0];
    
    for(byte a=0;a<LastStoredPacketPosition-1;a++){
      Packets[a]=Packets[a+1];
    }
    LastStoredPacketPosition--;
    return true;
  }
  return false;
};

void DCCPacketStack::Clear(void){
  StoredPackets=0;
  LastStoredPacketPosition=1;
};

// ---------------------------------------------------------------------------------------------------------------------------------------------------------------
// Main library class 
// ---------------------------------------------------------------------------------------------------------------------------------------------------------------


bool DHDCC::GlobalFN(byte InFN){

    if(InFN<16){
        if(bitRead(GlobalFn1,InFN)==1){
          return true;
        }
        return false;
    }else if(InFN<29){
        if(bitRead(GlobalFn2,InFN-16)==1){
          return true;
        }
        return false;
    }
   return false;  
};

// decorator 
bool DHDCC::GetFnState(byte InFN){

    return GlobalFN(InFN);

};

bool DHDCC::GlobalFNSet(byte InFN, byte InState){

    if(InFN<16){
        if(InState){
          bitSet(GlobalFn1,InFN);
        }else{
          bitClear(GlobalFn1,InFN);
        }
        return true;
    }else if(InFN<29){
        if(InState){
          bitSet(GlobalFn2,InFN-16);
        }else{
          bitClear(GlobalFn2,InFN-16);
        }
        return true;
    }
   return false;  
};

byte DHDCC::GetSpeed(){
    return GlobalSpeed;
};

byte DHDCC::GetDirection(){
    return GlobalDirection;
};

void DHDCC::DecoderSetup(byte InPin){

   pinMode(InPin, INPUT_PULLUP);  // DCC signal input pin 2

   attachInterrupt(digitalPinToInterrupt(InPin),DCC_ISR,CHANGE); // attach interrupt ot dcc digital input
// ---------------------------------------------------------------------------------------------------------------------------------------------------------------
// CV Setup
// ---------------------------------------------------------------------------------------------------------------------------------------------------------------
  
  DHDCC_CV1=EEPROM.read(1); // read stored values from eeprom
  DHDCC_CV7=EEPROM.read(7); // 
  DHDCC_CV8=EEPROM.read(8); // 
  DHDCC_CV29=EEPROM.read(29); // 

// ---------------------------------------------------------------------------------------------------------------------------------------------------------------
// Service mode setup
// ---------------------------------------------------------------------------------------------------------------------------------------------------------------

  DHDCC_ServiceMode=false; // initially we're in operation mode
  
// ---------------------------------------------------------------------------------------------------------------------------------------------------------------
// Catcher setup
// ---------------------------------------------------------------------------------------------------------------------------------------------------------------

  DHDCC_PreambleBitsCounter=0; // DHDCC_Reset preamble bits counter

  for(byte i=0;i<DHDCC_MaxPacketLength;i++){
    DHDCC_TempPacket[i]=0; // DHDCC_Reset temp packet
  }
  DHDCC_TempPacketLength=0;
  DHDCC_DataBitsCounter=0; // DHDCC_Reset bits counter
  DHDCC_DatabytesCounter=0; // DHDCC_Reset bytes counter

  DHDCC_ParserStage=DHDCC_WaitPreamble; // set stage to DHDCC_WaitPreamble
  
  DHDCC_DHDCC_ResetPacketsCounter=0;

// ---------------------------------------------------------------------------------------------------------------------------------------------------------------
// DHDCC_Reset DHDCC_Stop DHDCC_Idle flags setup
// 
  DHDCC_Reset=false;
  DHDCC_Stop=false;
  DHDCC_Idle=false;

// ---------------------------------------------------------------------------------------------------------------------------------------------------------------
// DHDCC_Reset DHDCC_Stop DHDCC_Idle flags setup
// 
  GlobalSpeed=0;        // speed 256 steps? but we use only 128 steps for now
  GlobalDirection=0;    // direction 0/1
  GlobalFn1=0;   // function states FL,F1,F2,F3 - F15
  GlobalFn2=0;

// ---------------------------------------------------------------------------------------------------------------------------------------------------------------
// DHDCC_Reset CVAI
// 
CVAI_LAST_DHDCC_ADDRESS=0;
CVAI_LAST_DATA=0;
CVAI_UPDATE=false;

};

void DHDCC::ProcessPackets(void){

  DCCPacket TPacket;
  
  if(PacketStack.PopPacket(&TPacket)){
    // filter adress partition
    // DHDCC_ADDress 
    // 00000000 (0): Broadcast DHDCC_ADDress
    // DHDCC_ADDresses 00000001-01111111 (1-127)(inclusive): Multi-Function decoders with 7 bit DHDCC_ADDresses
    // DHDCC_ADDresses 10000000-10111111 (128-191)(inclusive): Basic Accessory Decoders with 9 bit DHDCC_ADDresses and Extended Accessory Decoders with 11-bit DHDCC_ADDresses
    // DHDCC_ADDresses 11000000-11100111 (192-231)(inclusive): Multi Function Decoders with 14 bit DHDCC_ADDresses
    // DHDCC_ADDresses 11101000-11111110 (232-254)(inclusive): Reserved for Future Use
    // DHDCC_ADDress 11111111 (255): DHDCC_Idle Packet

    if(TPacket.GetDHDCC_ADDress()==0){
      // broadcast
    }

    if(TPacket.GetDHDCC_ADDress()!=255 && TPacket.GetDHDCC_ADDress()!=0){
      //Serial.print(TPacket.GetData(0));
      //Serial.print(" ");
      //Serial.print(TPacket.GetData(1));
      //Serial.print(" ");
      //Serial.print(TPacket.GetData(2));
      //Serial.print(" ");
      //Serial.println(TPacket.GetData(3));
      //Serial.println(TPacket.GetData(1)>>5);       
      
    }
//
// ---------------------------------------------------------------------------------------------------------------------------------------------------------------
// Multi Function Digital Decoders
// ---------------------------------------------------------------------------------------------------------------------------------------------------------------
#ifdef DHDCC_MFDD
// ---------------------------------------------------------------------------------------------------------------------------------------------------------------
// Advanced Operations Instruction (001)
// ---------------------------------------------------------------------------------------------------------------------------------------------------------------
    
    if((TPacket.GetData(1)>>5)==1){
      if((TPacket.GetData(1)&31)==31){
        // 11111 : 128 speed step instruction (126+DHDCC_Stop+emergency DHDCC_Stop)
        // bit 7 direction(1-forward,0-reverse), bits 0-6 speed
        GlobalDirection=bitRead(TPacket.GetData(2),7);
        //Serial.println(TPacket.GetData(2),BIN);
        if((TPacket.GetData(2)&127)==0 ||(TPacket.GetData(2)&127)==1){
          GlobalSpeed=0;  
        }else{
          GlobalSpeed=(TPacket.GetData(2)&127)-1;
        }
      }else
      if((TPacket.GetData(1)&31)==30){
        // 11110 : Restricted Speed Step Instruction
        
        if(bitRead(DHDCC_CV29,1)){
          // 28 steps speed control
          if((TPacket.GetData(1)&31)==0 || (TPacket.GetData(1)&31)==16){
            GlobalSpeed=0;
          }else
          if((TPacket.GetData(1)&31)==1 || (TPacket.GetData(1)&31)==17){
            GlobalSpeed=0;
          }else
          if((TPacket.GetData(1)&31)>15){
            GlobalSpeed=((TPacket.GetData(1)&31)-16)*2-2;
          }else{
            GlobalSpeed=((TPacket.GetData(1)&15)*2)-3;
          }
        }else{
          // 14 steps speed control
          //Serial.println(TPacket.GetData(1),BIN);
          if((TPacket.GetData(1)&15)==0){
            GlobalSpeed=0; 
          }else{
            GlobalSpeed=(TPacket.GetData(1)&15)-1;            
          }
          GlobalFNSet(0,bitRead(TPacket.GetData(1),4));         
        }    
      }else
      if((TPacket.GetData(1)&31)==29){
        // 11101 : Analog Function Group
        // TPacket.GetData(2) Analog Function Output (0-255)
        // TPacket.GetData(3) Analog Function Data (0-255)
      }
    }else
    
// ---------------------------------------------------------------------------------------------------------------------------------------------------------------
// Speed and Direction Instructions (010 and 011)
// ---------------------------------------------------------------------------------------------------------------------------------------------------------------
    if(((TPacket.GetData(1)>>5)==2) || ((TPacket.GetData(1)>>5)==3)){
      // 010 - reverse
      // 011 - forward      
      if((TPacket.GetData(1)>>5)==2){
        GlobalDirection=0;
      }else{
        GlobalDirection=1;
      }
      // DHDCC_CV29 bit 1 = 1 speed size 5 bits, else bit#4 controls headlight (FL)
      if(bitRead(DHDCC_CV29,1)){
        // 28 steps speed control
        if((TPacket.GetData(1)&31)==0 || (TPacket.GetData(1)&31)==16){
          GlobalSpeed=0;
        }else
        if((TPacket.GetData(1)&31)==1 || (TPacket.GetData(1)&31)==17){
          GlobalSpeed=0;
        }else
        if((TPacket.GetData(1)&31)>15){
          GlobalSpeed=((TPacket.GetData(1)&31)-16)*2-2;
        }else{
          GlobalSpeed=((TPacket.GetData(1)&15)*2)-3;
        }
      }else{
         // 14 steps speed control
         //Serial.println(TPacket.GetData(1),BIN);
         if((TPacket.GetData(1)&15)==0){
            GlobalSpeed=0; 
         }else{
            GlobalSpeed=(TPacket.GetData(1)&15)-1;            
         }
         GlobalFNSet(0,bitRead(TPacket.GetData(1),4));         
      }    
    }else
    
// ---------------------------------------------------------------------------------------------------------------------------------------------------------------
// Function Group One Instruction (100) controls FL if DHDCC_CV29 bit 1 is set, F1-F4
// ---------------------------------------------------------------------------------------------------------------------------------------------------------------
    if((TPacket.GetData(1)>>5)==4 ){

      //Serial.println("--3");


      
      GlobalFNSet(1,bitRead(TPacket.GetData(1),0)); //  F1 on/off
      GlobalFNSet(2,bitRead(TPacket.GetData(1),1)); //  F2 on/off
      GlobalFNSet(3,bitRead(TPacket.GetData(1),2)); //  F3 on/off
      GlobalFNSet(4,bitRead(TPacket.GetData(1),3)); //  F4 on/off
      if(!bitRead(DHDCC_CV29,1)){
        GlobalFNSet(0,bitRead(TPacket.GetData(1),4)); //  FL on/off
      }
    }else
    
// ---------------------------------------------------------------------------------------------------------------------------------------------------------------
// Function Group Two Instruction (101) controls F5-F12
// ---------------------------------------------------------------------------------------------------------------------------------------------------------------
    if((TPacket.GetData(1)>>5)==5){
      //Serial.println("--4");
      if(bitRead(TPacket.GetData(1),4)){
        // F5-F8  
        GlobalFNSet(5,bitRead(TPacket.GetData(1),0)); //  F5 on/off
        GlobalFNSet(6,bitRead(TPacket.GetData(1),1)); //  F6 on/off
        GlobalFNSet(7,bitRead(TPacket.GetData(1),2)); //  F7 on/off
        GlobalFNSet(8,bitRead(TPacket.GetData(1),3)); //  F8 on/off        
      }else{
        // F9-F12
        GlobalFNSet(9,bitRead(TPacket.GetData(1),0)); //  F9 on/off
        GlobalFNSet(10,bitRead(TPacket.GetData(1),1)); //  F10 on/off
        GlobalFNSet(11,bitRead(TPacket.GetData(1),2)); //  F11 on/off
        GlobalFNSet(12,bitRead(TPacket.GetData(1),3)); //  F12 on/off        
      }     
    }else
// ---------------------------------------------------------------------------------------------------------------------------------------------------------------
// Feature Expansion Instruction (110) 
// ---------------------------------------------------------------------------------------------------------------------------------------------------------------
     if((TPacket.GetData(1)>>5)==6){
           //Serial.println("--5");

       // don't know how to use it with Pro Mini, but....
       // CCCCC = 00000 - Binary State Control Instruction long form
       /*
         if((TPacket.GetData(1)<<3)>>3==0){         
         bool TempBinarySate=bitRead(7,TPacket.GetData(2));
         unsigned int TempDHDCC_ADDress=(TPacket.GetData(3)<<8;
         TempDHDCC_ADDress=TempDHDCC_ADDress+((TPacket.GetData(2)<<1)>>1);

         if(TempDHDCC_ADDress==0){
           // broadcast! we must set or clear all 32767 states to TemBinaryState
         }                  
       }else
       // CCCCC = 11101 - Binary State Control Instruction short form
       if((TPacket.GetData(1)<<3)>>3==29){
          bool TempBinarySate=bitRead(7,TPacket.GetData(2));
          unsigned int TempDHDCC_ADDress=((TPacket.GetData(2)<<1)>>1);
          if(TempDHDCC_ADDress==0){
           // broadcast! we must set or clear all 32767 states to TemBinaryState
          }       
       }else
       */
       // CCCCC = 11110 - F13-F20 Function Control
       if((TPacket.GetData(1)&31)==30){
         GlobalFNSet(13,bitRead(TPacket.GetData(2),0)); //  F13 on/off
         GlobalFNSet(14,bitRead(TPacket.GetData(2),1)); //  F14 on/off
         GlobalFNSet(15,bitRead(TPacket.GetData(2),2)); //  F15 on/off
         GlobalFNSet(16,bitRead(TPacket.GetData(2),3)); //  F16 on/off
         GlobalFNSet(17,bitRead(TPacket.GetData(2),4)); //  F17 on/off
         GlobalFNSet(18,bitRead(TPacket.GetData(2),5)); //  F18 on/off
         GlobalFNSet(19,bitRead(TPacket.GetData(2),6)); //  F19 on/off
         GlobalFNSet(20,bitRead(TPacket.GetData(2),7)); //  F20 on/off         
       }else
       // CCCCC = 11111 - F21-F28 Function Control
       if((TPacket.GetData(1)&31)==31){
         GlobalFNSet(21,bitRead(TPacket.GetData(2),0)); //  F21 on/off         
         GlobalFNSet(22,bitRead(TPacket.GetData(2),1)); //  F22 on/off         
         GlobalFNSet(23,bitRead(TPacket.GetData(2),2)); //  F23 on/off         
         GlobalFNSet(24,bitRead(TPacket.GetData(2),3)); //  F24 on/off         
         GlobalFNSet(25,bitRead(TPacket.GetData(2),4)); //  F25 on/off         
         GlobalFNSet(26,bitRead(TPacket.GetData(2),5)); //  F26 on/off         
         GlobalFNSet(27,bitRead(TPacket.GetData(2),6)); //  F27 on/off         
         GlobalFNSet(28,bitRead(TPacket.GetData(2),7)); //  F28 on/off         
       }
    }else

// ---------------------------------------------------------------------------------------------------------------------------------------------------------------
// Configuration Variable Access Instruction (111)
// ---------------------------------------------------------------------------------------------------------------------------------------------------------------
    if((TPacket.GetData(1)>>5)==7){
            //Serial.println("--6");

      
      // Configuration Variable Access Instruction - Short Form
      if(((TPacket.GetData(1)>>4)&15)==15){
        if((TPacket.GetData(1)&15)==2){
          // cccc = 0010 set CV23 
          EEPROM.write(23,TPacket.GetData(2));         
        }else
        if((TPacket.GetData(1)&15)==3){
          // cccc = 0011 set CV24 
          EEPROM.write(24,TPacket.GetData(2));
        }
        /*
        if((TPacket.GetData(1)&15)==9){
          // cccc = 1001 // LOCK DECODER INSTRUCTION (NMRA S-9.2.3, Appendix B), I think we don't need to implement it
        }*/
      }
      
      // Configuration Variable Access Instruction - Long Form
      if((TPacket.GetData(1)>>4)&14==14){
        // 11110CCVV
        // CC = 00 - Reserved
        // CC = 01 - Verify byte
        if((TPacket.GetData(1)&12)>>2==1){
          // verify byte
        }else
        if((TPacket.GetData(1)&12)>>2==3){
          // write byte with 10 bit adress - need to implement 2-nd packet catcher for this function!
          unsigned int TempCVDHDCC_ADDress=TPacket.GetData(1)&3;
          TempCVDHDCC_ADDress=(TempCVDHDCC_ADDress<<8)+TPacket.GetData(2);
          
          if(TempCVDHDCC_ADDress<DHDCC_EEPROM_LENGTH){
            if(CVAI_LAST_DHDCC_ADDRESS==TempCVDHDCC_ADDress && CVAI_LAST_DATA==TPacket.GetData(3)){
              EEPROM.write(CVAI_LAST_DHDCC_ADDRESS,CVAI_LAST_DATA);              
              CVAI_LAST_DHDCC_ADDRESS=0;
              CVAI_LAST_DATA=0;
              CVAI_UPDATE=false;
            }else{
              CVAI_LAST_DHDCC_ADDRESS=TempCVDHDCC_ADDress;
              CVAI_LAST_DATA=TPacket.GetData(3);
              CVAI_UPDATE=true;              
            }              
          }    
        }else
        if((TPacket.GetData(1)&12)>>2==2){
          // bit manipulation 
          unsigned int TempCVDHDCC_ADDress=TPacket.GetData(1)&3;
          TempCVDHDCC_ADDress=(TempCVDHDCC_ADDress<<8)+TPacket.GetData(2);
          
          if((TPacket.GetData(3)>>5)==7){
            byte TempBitOperation=(TPacket.GetData(3)&16);
            byte TempBitPosition=(TPacket.GetData(3)&7);
            byte TempBitContent=(TPacket.GetData(3)&8);
            if(TempBitOperation==0){
              // verify bit
              // not implemented yet
            }else{
              // write bit
              if(TempCVDHDCC_ADDress<DHDCC_EEPROM_LENGTH){
                byte TempData=EEPROM.read(TempCVDHDCC_ADDress);
                bitWrite(TempData,TempBitPosition,TempBitContent);
              }
            }  
          }
        }
      }
    }
    
    if(!CVAI_UPDATE){
      CVAI_LAST_DHDCC_ADDRESS=0;
      CVAI_LAST_DATA=0;    
    }
#endif    


// ---------------------------------------------------------------------------------------------------------------------------------------------------------------
// Accessory Digital Decoder Packet Formats    
// ---------------------------------------------------------------------------------------------------------------------------------------------------------------
#ifdef DHDCC_ADD

    // Basic Accessory Decoder Packet Format
    if(((TPacket.GetData(1)>>6)&2)==2 && ((TPacket.GetData(2)>>7)==1)){
      
    
    
    }else
    
    // Extended Accessory Decoder Control Packet Format
    if(((TPacket.GetData(1)>>6)&2)==2 && (TPacket.GetData(2)>>7)==0){

    }
#endif


  }



// ---------------------------------------------------------------------------------------------------------------------------------------------------------------
// process DHDCC_ServiceMode - need to rewrite?
// ---------------------------------------------------------------------------------------------------------------------------------------------------------------

  if(DHDCC_ServiceMode){
    // Direct mode (112)
    if((TPacket.GetDHDCC_ADDress()&112)==112 && TPacket.GetDHDCC_ADDress()!=255){
      // test for write CV
      if((TPacket.GetDHDCC_ADDress()&12)==12){
        // compose 10 bits DHDCC_ADDress (DHDCC_CV1 DHDCC_ADDress is 0, CV2 - 1 etc)
        unsigned int TDHDCC_ADDress=(TPacket.GetDHDCC_ADDress()&3)<<8;
        TDHDCC_ADDress=TDHDCC_ADDress+TPacket.GetData(1);        
        // next filter our CV
        switch (TDHDCC_ADDress+1){
          case 1:
            DHDCC_CV1=TPacket.GetData(2);
            EEPROM.write(TDHDCC_ADDress,DHDCC_CV1);
            Serial.print("CV");
            Serial.print(TDHDCC_ADDress+1,DEC);
            Serial.print("=");
            Serial.println(DHDCC_CV1,DEC);
            delay(500);
            PacketStack.Clear();
            DHDCC_ServiceMode=false;
          break;        
          case 29:
            DHDCC_CV29=TPacket.GetData(2);
            EEPROM.write(TDHDCC_ADDress,DHDCC_CV1);
            Serial.print("CV");
            Serial.print(TDHDCC_ADDress+1,DEC);
            Serial.print("=");
            Serial.println(DHDCC_CV29,DEC);
            delay(500);
            PacketStack.Clear();
            DHDCC_ServiceMode=false;
          break;        
        }        
      }
    }    
  }else
  
  // process DHDCC_Reset
  if(DHDCC_Reset){
    DHDCC_DHDCC_ResetPacketsCounter++;
    if(DHDCC_DHDCC_ResetPacketsCounter>=3){
      // Enter to service mode
      DHDCC_ServiceMode=true;
      PacketStack.Clear();
    }
  }else   
  // process DHDCC_Stop
  if(DHDCC_Stop){
  }else
  // process DHDCC_Idle
  if(DHDCC_Idle){
  }else{
    // process normal packet
    // 
    DHDCC_DHDCC_ResetPacketsCounter=0;  
  };
}

void DCC_ISR(){
  //  DHDCC_Intervals for ProMini  ~ 64 & 120
  noInterrupts();
  if(digitalRead(2)==HIGH){
    DHDCC_StartTime=micros();
  }else{
    DHDCC_Interval=micros()-DHDCC_StartTime;
    byte rBit=0;
    // get next bit
    // if DHDCC_Interval <116 (<58ms*2) bit="1", if more (>>95ms*2) bit="0"
    if (DHDCC_Interval<=116){
      rBit=1;
    }else{
      rBit=0;
    }
    switch (DHDCC_ParserStage){
      // first stage - wait preamble (at least 10 "1" bits) 
      case DHDCC_WaitPreamble:
      {
        if(rBit==1){
          DHDCC_PreambleBitsCounter++;
          if(DHDCC_PreambleBitsCounter==10){
            DHDCC_ParserStage=DHDCC_WaitPacketStartBit;
            DHDCC_PreambleBitsCounter=0;
          }
        }else{
          DHDCC_PreambleBitsCounter=0;  
        }          
      }
      break;  
      // wait for PacketStartBit (0)
      case DHDCC_WaitPacketStartBit:
      {
        if(rBit==0){
          DHDCC_ParserStage=DHDCC_ReadDHDCC_ADDress;
          DHDCC_DataBitsCounter=0;
          DHDCC_DatabytesCounter=0;
        }
      }
      break;
      // read DHDCC_ADDress byte into DHDCC_TempPacket[0]
      case DHDCC_ReadDHDCC_ADDress:
      {
        //set bits
        if(rBit==1){
          bitSet(DHDCC_TempPacket[DHDCC_DatabytesCounter],7-DHDCC_DataBitsCounter);
        } else {
          bitClear(DHDCC_TempPacket[DHDCC_DatabytesCounter],7-DHDCC_DataBitsCounter);
        }          
        DHDCC_DataBitsCounter++;
        // it was last DHDCC_ADDress bit, wait for DataStartBit
        if(DHDCC_DataBitsCounter==8){
          DHDCC_ParserStage=DHDCC_DHDCC_WaitDataStartBit;
          DHDCC_DataBitsCounter=0;
          DHDCC_DatabytesCounter++;
        }
      }
      break;
      // Wait for data start bit
      case DHDCC_DHDCC_WaitDataStartBit:
      {
        if(rBit==0){
          DHDCC_ParserStage=DHDCC_ReDHDCC_ADData;
        }else{
          
          if(DHDCC_TempPacket[0]==DHDCC_CV1 || DHDCC_ServiceMode){
            // normal packet        
            // check for consistency              
            byte Check=DHDCC_TempPacket[0];
            for(byte a=1;a<DHDCC_DatabytesCounter-1;a++){
              Check=Check^DHDCC_TempPacket[a];
            }
            if(Check==DHDCC_TempPacket[DHDCC_DatabytesCounter-1]){
              //good packet, do something
              DCCPacket TPacket;
              TPacket.SetDHDCC_ADDress(&DHDCC_TempPacket[0]);
              for (byte a=0;a<DHDCC_DatabytesCounter;a++){
                TPacket.PushData(&DHDCC_TempPacket[a]);
              }
              PacketStack.PushPacket(&TPacket);
            }
          }
          else if(DHDCC_TempPacket[0]==0 && DHDCC_DatabytesCounter==3){
            // possble DHDCC_Reset or DHDCC_Stop packet, check it
            if(DHDCC_TempPacket[1]==0 && DHDCC_TempPacket[2]==0){
                // DHDCC_Reset packet received
                DHDCC_Reset=true;
            }
            if(((DHDCC_TempPacket[1]&65)==65) && ((DHDCC_TempPacket[0]^DHDCC_TempPacket[1])==DHDCC_TempPacket[2])){
                // DHDCC_Stop packet received
                DHDCC_Stop=true;
            }
          }else if(DHDCC_TempPacket[0]==1 && DHDCC_DatabytesCounter==3){
            // possible DHDCC_Idle packet
            if(DHDCC_TempPacket[1]==0 && DHDCC_TempPacket[2]==1){
                // DHDCC_Idle packet received!!!
                DHDCC_Idle=true;
            }
          }
          DHDCC_ParserStage=DHDCC_WaitPreamble; 
        }
      }
      break;
      case DHDCC_ReDHDCC_ADData:
      {
        if(rBit==1){
          bitSet(DHDCC_TempPacket[DHDCC_DatabytesCounter],7-DHDCC_DataBitsCounter);
        } else {
          bitClear(DHDCC_TempPacket[DHDCC_DatabytesCounter],7-DHDCC_DataBitsCounter);
        }          
        DHDCC_DataBitsCounter++;
        if(DHDCC_DataBitsCounter==8){
          DHDCC_ParserStage=DHDCC_DHDCC_WaitDataStartBit;
          DHDCC_DataBitsCounter=0;
          DHDCC_DatabytesCounter++;
        }
      }
      break;
    }
  }
  interrupts();
};



/*
void print_binary(int v, int num_places)
{
    int mask=0, n;
    for (n=1; n<=num_places; n++)
    {
        mask = (mask << 1) | 0x0001;
    }
    v = v & mask;  // truncate v to specified number of places
    while(num_places)
    {
        if (v & (0x0001 << num_places-1))
        {
             Serial.print("1");
        }
        else
        {
             Serial.print("0");
        }
       --num_places;
        if(((num_places%4) == 0) && (num_places != 0))
        {
            //Serial.print("_");
        }
    }
}*/


   /*
    #ifdef MF7BIT
    else if(TPacket.GetDHDCC_ADDress()>=1 && TPacket.GetDHDCC_ADDress()<=127){
      // Multi-Function decoders with 7 bit DHDCC_ADDresses
    }
    #endif
    #ifdef BA9BIT
    else if(TPacket.GetDHDCC_ADDress()>=128 && TPacket.GetDHDCC_ADDress()<=191){
      // Basic accessory decoders (9bit DHDCC_ADDress) && extended accesory decoders with 11-bit DHDCC_ADDress
    }
    #endif
    #ifdef MF14BIT
    else if(TPacket.GetDHDCC_ADDress()>=192 && TPacket.GetDHDCC_ADDress()<=231){
      // Multi function decoder 14 bit DHDCC_ADDress
    }
    #endif    
    else if(TPacket.GetDHDCC_ADDress()==255){
      // DHDCC_Idle Packet
    }

*/


